﻿-----------------------------------------------------
-- lua 파일이 여러개인 경우
-- 애드온명, 애드온내공유테이블이 ...으로 전달됨
-----------------------------------------------------
local addonName, addonLocalDB = ...;

--[[
케릭별 전문기술
기계공학   이진득성기사
마법부여   이진득성기사
요리       이진득성기사
낚시       이진득성기사
채광       이진득
연금술     이진득악사
약초채집   이진득악사
재봉술     이진득흑마법사
주문각인   이진득죽기
보석세공   이진득기사
가죽세공   이진득기사
무두질     이진득인간전사
대장기술   이진득인간전사
--]]

--[[
----------------------------
ID	Name
----------------------------
164 Blacksmithing   대장기술
165 Leatherworking  가죽세공
171 Alchemy         연금술
182 Herbalism       약초채집
185 Cooking         요리
186 Mining          채광
197 Tailoring       재봉술
202 Engineering     기계공학
333 Enchanting      마법부여
356 Fishing         낚시
393 Skinning        무두질
755 Jewelcrafting   보석세공
773 Inscription     주문각인
794 Archaeology     고고학 (주의 전문기술 없음)
--]]
local LJDProfessionSortedKeyList = {
    202, -- Engineering     기계공학
    333, -- Enchanting      마법부여
    185, -- Cooking         요리
    171, -- Alchemy         연금술
    197, -- Tailoring       재봉술
    182, -- Herbalism       약초채집
    186, -- Mining          채광
    356, -- Fishing         낚시
    393, -- Skinning        무두질
    794, -- Archaeology     고고학

    164, -- Blacksmithing   대장기술
    165, -- Leatherworking  가죽세공
    755, -- Jewelcrafting   보석세공
    773, -- Inscription     주문각인
};

local MAX_DISPLAY_RECIPE_NUM = 15; -- 툴팁에 표현될 최대 전문기술 갯수

-- 숙련도별 레시피 Color
local LJDTradeSkillTypeColor = {
    [0]             = { r = 1.00, g = 0.50, b = 0.25,   font = GameFontNormalLeftOrange },
    [1]             = { r = 1.00, g = 1.00, b = 0.00,   font = GameFontNormalLeftYellow },
    [2]             = { r = 0.25, g = 0.75, b = 0.25,   font = GameFontNormalLeftLightGreen },
    [3]             = { r = 0.50, g = 0.50, b = 0.50,   font = GameFontNormalLeftGrey },
    [4]             = { r = 1.00, g = 0.82, b = 0,      font = GameFontNormalLeft },
    [5]             = { r = 1.00, g = 0.82, b = 0,      font = GameFontNormalLeft },
    [6]             = { r = 0.96, g = 0.96, b = 0.96,   font = GameFontNormalLeftGrey },
    [7]             = { r = 0.96, g = 0.96, b = 0.96,   font = GameFontNormalLeftGrey },
    [8]             = { r = 0.96, g = 0.96, b = 0.96,   font = GameFontNormalLeftGrey },
    [9]             = { r = 0.96, g = 0.96, b = 0.96,   font = GameFontNormalLeftGrey },
    
    optimal         = { r = 1.00, g = 0.50, b = 0.25,   font = GameFontNormalLeftOrange },
    medium          = { r = 1.00, g = 1.00, b = 0.00,   font = GameFontNormalLeftYellow },
    easy            = { r = 0.25, g = 0.75, b = 0.25,   font = GameFontNormalLeftLightGreen },
    trivial         = { r = 0.50, g = 0.50, b = 0.50,   font = GameFontNormalLeftGrey },
    header          = { r = 1.00, g = 0.82, b = 0,      font = GameFontNormalLeft },
    subheader       = { r = 1.00, g = 0.82, b = 0,      font = GameFontNormalLeft },
    nodifficulty    = { r = 0.96, g = 0.96, b = 0.96,   font = GameFontNormalLeftGrey },
    
    -- 이하 진득 추가
    AlertRecipe     = { r = 1.00, g = 0.00, b = 1.00,   font = GameFontNormalLeft },    -- AlertRecipe 표현색, 보라
    NormalColor     = { r = 1.00, g = 0.82, b = 0.00,   font = GameFontNormalLeft },    -- 기본 표현색(NORMAL_FONT_COLOR), 약간 어두운 노랑
    ItemCount       = { r = 1.00, g = 1.00, b = 1.00,   font = GameFontNormalLeft },    -- 보유갯수 표현색, 흰색
};

local categoryColor      = LJDTradeSkillTypeColor.NormalColor;      -- 카테고리 표현색, 툴팁기본색을 사용하지 않고 색 지정
local itemCountColor     = LJDTradeSkillTypeColor.ItemCount;        -- 아이템보유갯수 표현색
-- rgb => argbHex 변환( a=ff 고정)
local countColorArgbHex  = ("ff%.2x%.2x%.2x"):format(itemCountColor.r * 255, itemCountColor.g * 255, itemCountColor.b * 255);

-- 만들고자 하는 레시피(눈에 띄게 보라색 표현될 레시피)
local AlertRecipeList = {
    "고블린 글라이더 조립도구",
    "심해 가방",
    --"수놓은 심해 가방",
};

local unitsOfNumber = "개";
-- 한국이 아니면 단위없애기
if (GetLocale() ~= "koKR") then
    unitsOfNumber = "";
end

-- 실행된 와우가 클래식인지 체크용 변수
local isClassic = (WOW_PROJECT_ID == WOW_PROJECT_CLASSIC);

-- 보유갯수 표현 설정변경 토글명령어
--[[
/LJDProfession ShowNumberOfItems
/LJDProfession ReloadDefaultDB
--]]
SLASH_LJDProfession1 = "/LJDProfession";
SlashCmdList.LJDProfession = function(msg)
    if(msg == "ShowNumberOfItems") then
        LJDProfessionConfDB["IsShowNumberOfItems"] = not LJDProfessionConfDB["IsShowNumberOfItems"];
        print("LJDProfession ShowNumberOfItems : ", LJDProfessionConfDB["IsShowNumberOfItems"]);
    elseif(msg == "ReloadDefaultDB") then
        LJDProfessionDB = LJDProfessionDB_Default;
        print("LJDProfession Reloaded !!");
    end
end

local frame = CreateFrame("FRAME");
frame:SetScript('OnEvent', function(self, event, ...)
    return self:Events(event, ...);
end);

--frame:RegisterEvent("TRADE_SKILL_DATA_SOURCE_CHANGING");
frame:RegisterEvent("TRADE_SKILL_DATA_SOURCE_CHANGED");
--frame:RegisterEvent("TRADE_SKILL_LIST_UPDATE");
frame:RegisterEvent("TRADE_SKILL_DETAILS_UPDATE");
--frame:RegisterEvent("TRADE_SKILL_NAME_UPDATE");
--frame:RegisterEvent("SKILL_LINES_CHANGED");
--frame:RegisterEvent("TRIAL_STATUS_UPDATE");
--frame:RegisterEvent("TRADE_SKILL_CLOSE");
--frame:RegisterEvent("GARRISON_TRADESKILL_NPC_CLOSED");
frame:RegisterEvent("PLAYER_ENTERING_WORLD");
frame:RegisterEvent("ADDON_LOADED");
    
function frame:Events(event, ...)
    local ll = {...};
    --DebugPrint(event);
    --DebugPrint(self);
    --DebugPrintTable(self);
    
    if( event == "PLAYER_ENTERING_WORLD" ) then
        self:PLAYER_ENTERING_WORLD();
    elseif( event == "ADDON_LOADED" ) then
        self:ADDON_LOADED(...);
    else
        --DebugPrint(event);
        LJDSaveTradeSkillList();
    end
end

function frame:ADDON_LOADED(argAddonName)
    --print(argAddonName);
    if argAddonName ~= "LJDProfession" then return end;
    
    -- 이전버젼 사용자들을 감안하여 "기계공학" 등 name 형태가
    -- 있으면 초기화처리
    -- For OLD Version user
    if( LJDProfessionDB and LJDProfessionDB["기계공학"] ) then
        LJDProfessionDB = nil;
    end
    
    -- 이진득 전문기술 데이터 기본값 Load
    if LJDProfessionDB == nil then
        LJDProfessionDB = LJDProfessionDB_Default;
    end

    -- 이진득 전문기술 애드온설정 기본값 초기화
    if LJDProfessionConfDB == nil then
        LJDProfessionConfDB = {
            ["IsShowNumberOfItems"] = true,
        };
    end
    
    -- 플레이어의 전문기술을 채팅창에 출력
    --LJDPrintPlayerProfessions();
    
        
    -- 전문기술창 이동가능하게 변경
    -- ; LJDSpellInterrupt의 Util 사용.
    if( ChangeMoveableFrame ) then
        ChangeMoveableFrame(ProfessionsFrame);
    end

end

function LJDPrintPlayerProfessions()
    -- 클래식이면 내 전문기술 출력안함
    if( isClassic ) then
        return;
    end
    
    --원하는 기능
    --재료 아이템을 마우스오버했을때, 모든 전문 기술에서 어떤 레시피의 재료임을 보여주고자 한다.
    --Ex) 옷감 마우스오버시, [재봉술] 어깨 옷 제작재료..

    local prof1, prof2, archaeology, fishing, cooking, firstAid = GetProfessions();
    if( prof1 ) then
        local name, icon, skillLevel, maxSkillLevel, numAbilities, spelloffset, skillLine, skillModifier, specializationIndex, specializationOffset = GetProfessionInfo(prof1);
        print("Profession 1 : " .. name);
    end
    if( prof2 ) then
        local name, icon, skillLevel, maxSkillLevel, numAbilities, spelloffset, skillLine, skillModifier, specializationIndex, specializationOffset = GetProfessionInfo(prof2);
        print("Profession 2 : " .. name);
    end
end

function frame:PLAYER_ENTERING_WORLD()
    -- 플레이어의 전문기술을 채팅창에 출력
    LJDPrintPlayerProfessions();
end

--[[
HighScore = { Robin = 8, Jon = 10, Max = 11 }

-- basic usage, just sort by the keys
for k,v in spairs(HighScore) do
    print(k,v)
end
--> Jon     10
--> Max     11
--> Robin   8

-- this uses an custom sorting function ordering by score descending
for k,v in spairs(HighScore, function(t,a,b) return t[b] < t[a] end) do
    print(k,v)
end
--> Max     11
--> Jon     10
--> Robin   8
--]]
-- 단순 Key 값으로 정렬기능
-- 진득주)
-- 1. table.sort를 이용한 정렬은 단순 List 형태(index-value)만 가능하다. ex. {"일","이","삼"}
-- 2. table.sort는 List형이 아니면 sort함수를 지정하더라도 반응하지 않는다.
-- 3. table(key-value)에 순차적으로 값을 넣는다고 해도 순서는 보장 받지 못한다.
-- 종합적으로, 순서보장을 위해서는 List형태(index-value)를 사용하고,
-- 저장된 데이터는 가공할수 없으므로 아래처럼 순차적으로 꺼내는 함수를 이용함.
function LJDProfessionSortPairs(t, order)
    -- collect the keys
    local keys = {}
    
    for k in pairs(t) do keys[#keys+1] = k end

    -- if order function given, sort by it by passing the table and keys a, b,
    -- otherwise just sort the keys 
    if order then
        table.sort(keys, function(a,b) return order(t, a, b) end)
    else
        table.sort(keys)
    end

    -- return the iterator function
    local i = 0
    return function()
        i = i + 1
        if keys[i] then
            return keys[i], t[keys[i] ]
        end
    end
end

-- 아래 함수는 레시피의 출력순서를 전문기술별로 정한다.
-- table.sort 사용을 고려하였으나 table.sort 불안정함 문제로 사용안함.
function LJDProfessionSortPairsByKeyTable(orgTable, sortedKeyTable)

    if( not sortedKeyTable ) then
        sortedKeyTable = LJDProfessionSortedKeyList;    -- 전역으로 선언
    end

    --------------------------------------------
    -- iterator function Start
    --------------------------------------------
    local i = 0;
    local function LJDIterFunc()
        i = i + 1;
        
        -- 이경우는 있을수 없으나 안전빵
        if not sortedKeyTable[i] then
            return nil;
        end
        
        -- sortedKeyTable 에만 있는 경우
        -- orgTable 에서 순차적으로 찾는다.
        if( not orgTable[ sortedKeyTable[i] ] ) then
            for j=i, #sortedKeyTable do
                i = j;  -- i 증가(찾을때까지)
                if( orgTable[ sortedKeyTable[i] ] ) then
                    break;
                end
            end
        end

        -- orgTable 에서 찾아봤지만 없는 경우를 감안하여..ex. 고고학
        if not orgTable[ sortedKeyTable[i] ] then
            return nil;
        end

        -- row 리턴
        return sortedKeyTable[i], orgTable[ sortedKeyTable[i] ];
    end
    --------------------------------------------
    -- iterator function End
    --------------------------------------------
    return LJDIterFunc;
end

function LJDSaveTradeSkillList()

    -- 클래식이면 레시피정보 갱신 안함.(엄밀히는 API 없음으로 못함)
    if( isClassic ) then
        return;
    end

    if( not ProfessionsFrame:IsShown() ) then
        return;
    end

--DebugPrintTable(ProfessionsFrame.GetProfessionInfo());
    local professionInfo = ProfessionsFrame.GetProfessionInfo();
    
    parentSkillLineName = professionInfo.parentProfessionName;
    parentSkillLineID   = professionInfo.parentProfessionID;
    
    --DebugPrintTable(C_TradeSkillUI.GetProfessionInfoBySkillLineID(parentSkillLineID));
    
    
    -- 20221029 레시피갱신실패로 인해 저장하지 않도록 함.
    -- 10.0.0 API 삭제로 인해 레시피갱신실패
    -- GetRecipeNumReagents
    -- GetRecipeReagentInfo
    --if( 1 == 1 ) then return;end;

    if( not parentSkillLineID ) then
        return;
    end
    
    --
    --LJDProfessionDB = {}; -- 디버그용 기존정보 지우기(갱신필요)
    LJDProfessionDB = LJDProfessionDB or {}
    --LJDProfessionDB[parentSkillLineID] = {}; -- 갱신전 기존 내용 제거
    --
    
    local recipeIDs = C_TradeSkillUI.GetAllRecipeIDs();         -- 전체(한번은 전문기술 창을 열어야 함)
    --DebugPrintTable(recipeIDs);
    --print(#recipeIDs);
    local savedIndex = 0;
    local bSave = true;
    local recipeInfo = {};
    local deleteList = {}; -- 레시피DB삭제대상
--if( 1 == 1 ) then return; end;
    --[[

         Blizzard_ProfessionsRecipeTracker.lua 발췌
    
		local colorStyle = OBJECTIVE_TRACKER_COLOR["Normal"];
		local isRecraft = false;

		for _, recipeID in ipairs(C_TradeSkillUI.GetRecipesTracked()) do
			local recipeSchematic = C_TradeSkillUI.GetRecipeSchematic(recipeID, isRecraft);
			local block = self:GetBlock(recipeID);
			self:SetBlockHeader(block, recipeSchematic.name);

			for slotIndex, reagentSlotSchematic in ipairs(recipeSchematic.reagentSlotSchematics) do
				if reagentSlotSchematic.reagentType == Enum.CraftingReagentType.Basic then
					local reagent = reagentSlotSchematic.reagents[1];
					if reagent.itemID then
						local item = Item:CreateFromItemID(reagent.itemID);
						local itemName = item:GetItemName();
						local quantity = Professions.AccumulateReagentsInPossession(reagentSlotSchematic.reagents);
						local quantityRequired = reagentSlotSchematic.quantityRequired;
						local text = PROFESSIONS_TRACKER_REAGENT_FORMAT:format(PROFESSIONS_TRACKER_REAGENT_COUNT_FORMAT:format(quantity, quantityRequired), itemName)
						self:AddObjective(block, slotIndex, text, nil, nil, OBJECTIVE_DASH_STYLE_SHOW, colorStyle);
					end
				end
			end

			block:SetHeight(block.height);

			if ( ObjectiveTracker_AddBlock(block) ) then
				block:Show();
				self:FreeUnusedLines(block);
			else
				block.used = false;
				break;
			end
		end

    local recipeIDs_Tracked = C_TradeSkillUI.GetRecipesTracked(); -- 추적중인 레시피ID
    DebugPrintTable(recipeIDs_Tracked);
    --print(#recipeIDs_Tracked);


    --]]
    
    
    
    
    
    
    
    
    
    for idx = 1, #recipeIDs do
    --for idx = 1, 1 do

        bSave = true;   -- continue 대용
        recipeInfo = {};    -- ByRef 회피용 초기화
        
        -- 20201018 9.0.1 patch
        --C_TradeSkillUI.GetRecipeInfo(recipeIDs[idx], recipeInfo);
        recipeInfo = C_TradeSkillUI.GetRecipeInfo(recipeIDs[idx]);
--DebugPrint(idx ..":".. recipeInfo.name);
--DebugPrintTable(recipeInfo);
        --3 스타만 남기기.. 동작안함. 다른 방식으로 처리함.
        --recipeInfo = TradeSkillFrame.RecipeList:FindBestStarRankLinksForRecipe(recipeInfo);
        
        --function TradeSkillRecipeListMixin:FindBestStarRankLinksForRecipe(recipeInfo)
        -- 위 함수 개조하였으나 쓸일이 없음.
        --[[
            local startingRecipeInfo = recipeInfo;
            while startingRecipeInfo.previousRecipeID do
                C_TradeSkillUI.GetRecipeInfo(startingRecipeInfo.previousRecipeID, startingRecipeInfo);
            end

            local bestRecipeInfo = startingRecipeInfo;
            while bestRecipeInfo.nextRecipeID and bestRecipeInfo.learned do
                C_TradeSkillUI.GetRecipeInfo(bestRecipeInfo.nextRecipeID, bestRecipeInfo);
            end

            return bestRecipeInfo;
        --end
        --]]
        
        -- 추가로 저장할 데이터 추가
        local parentCategoryName = parentSkillLineName;   -- 카테고리초기값 skillLineName ex. 요리
        local parentCategoryID   = nil;
        local categoryData = C_TradeSkillUI.GetCategoryInfo(recipeInfo.categoryID);
        if categoryData then
            parentCategoryName = categoryData.name; -- 상위 카테고리, ex. 가벼운 식사
            parentCategoryID   = categoryData.parentCategoryID;

            if parentCategoryID then
                local parentCategoryData = C_TradeSkillUI.GetCategoryInfo(parentCategoryID);
                parentCategoryName = parentCategoryData.name;   -- 상위 카테고리, ex. 잔달라 요리법
            end
        end
        recipeInfo.parentCategoryID   = parentCategoryID;    -- 상위카테고리ID
        recipeInfo.parentCategoryName = parentCategoryName;  -- 상위카테고리명

        -- 20191214 추가
        recipeInfo.skillLineID   = parentSkillLineID;     -- 전문기술ID
        recipeInfo.skillLineName = parentSkillLineName;   -- 전문기술명

        recipeInfo.difficulty = recipeInfo.relativeDifficulty;

--DebugPrintTable(recipeInfo);

        local recipeSchematic = C_TradeSkillUI.GetRecipeSchematic(recipeIDs[idx], false);
        
        
        --[[
        --if( recipeSchematic) then
            --DebugPrint("Not Exist !!");
            --DebugPrintTable(recipeSchematic);
            
			for slotIndex, reagentSlotSchematic in ipairs(recipeSchematic.reagentSlotSchematics) do
				if reagentSlotSchematic.reagentType == Enum.CraftingReagentType.Basic then
					local reagent = reagentSlotSchematic.reagents[1];
					if reagent.itemID then
                        local item = Item:CreateFromItemID(reagent.itemID);
						local itemName = item:GetItemName();
						local quantity = Professions.AccumulateReagentsInPossession(reagentSlotSchematic.reagents);
						local quantityRequired = reagentSlotSchematic.quantityRequired;
                        
                        
                        --DebugPrint("   >"..tostring(reagent.itemID)..":"..tostring(itemName));
					end
				end
			end
        --end
        --]]
        
        

        --local numReagents = C_TradeSkillUI.GetRecipeNumReagents(recipeIDs[idx]);
        --DebugPrintTable(ProfessionsFrame);
        --DebugPrintTable(ProfessionsFrame.Pages);
        --DebugPrintTable(ProfessionsFrame.Pages[1]);
        --DebugPrintTable(ProfessionsFrame.Pages[2]);
        
        --DebugPrintTable(ProfessionsFrame.CraftingPage);
        --DebugPrintTable(ProfessionsFrame.SpecPage);
        
        --DebugPrintTable(ProfessionsFrame.CraftingPage.RecipeList);
        --DebugPrintTable(ProfessionsFrame.CraftingPage.SchematicForm);
        
        --DebugPrintTable(ProfessionsFrame.CraftingPage.SchematicForm.Details);
        --DebugPrintTable(ProfessionsFrame.CraftingPage.SchematicForm.Reagents);
        --DebugPrintTable(ProfessionsFrame.CraftingPage.SchematicForm.GetRecipeInfo());
        --DebugPrintTable(ProfessionsFrame.CraftingPage.SchematicForm.GetTransaction());
        --DebugPrintTable(ProfessionsFrame.CraftingPage.SchematicForm.NineSlice);

        --DebugPrintTable(ProfessionsFrame.CraftingPage.SchematicForm.transaction);
        --if( ProfessionsFrame.CraftingPage.SchematicForm.transaction ) then
        
            --local reagents = ProfessionsFrame.CraftingPage.SchematicForm.transaction:CreateCraftingReagentInfoTbl();
            --DebugPrintTable(reagents[1]);
            --DebugPrintTable(ProfessionsFrame.CraftingPage.SchematicForm.transaction.recipeID);
            --DebugPrintTable(ProfessionsFrame.CraftingPage.SchematicForm.transaction.reagentTbls);
            --DebugPrintTable(ProfessionsFrame.CraftingPage.SchematicForm.transaction.reagentTbls[1]);
            --DebugPrintTable(ProfessionsFrame.CraftingPage.SchematicForm.transaction.reagentTbls[2]);

            --DebugPrintTable(ProfessionsFrame.CraftingPage.SchematicForm.transaction.reagentTbls[1].allocations);
            --DebugPrintTable(ProfessionsFrame.CraftingPage.SchematicForm.transaction.reagentTbls[1].allocations.Enumerate());
            --DebugPrintTable(ProfessionsFrame.CraftingPage.SchematicForm.transaction.reagentTbls[1].allocations.allocs);
            --DebugPrintTable(ProfessionsFrame.CraftingPage.SchematicForm.transaction.reagentTbls[1].allocations.allocs[1]);
            --DebugPrintTable(ProfessionsFrame.CraftingPage.SchematicForm.transaction.reagentTbls[1].allocations.allocs[1].GetReagent());
            --DebugPrintTable(ProfessionsFrame.CraftingPage.SchematicForm.transaction.reagentTbls[1].allocations.allocs[1].reagent);
            --DebugPrintTable(ProfessionsFrame.CraftingPage.SchematicForm.transaction.reagentTbls[2].allocations.allocs);
            --DebugPrint(ProfessionsFrame.CraftingPage.SchematicForm.transaction.reagentTbls[2].allocations.allocs);
            --DebugPrintTable(ProfessionsFrame.CraftingPage.SchematicForm.transaction.reagentTbls[2].allocations.allocs.reagent);
            --DebugPrintTable(ProfessionsFrame.CraftingPage.SchematicForm.transaction.reagentTbls[1].reagentSlotSchematic);
            --DebugPrintTable(ProfessionsFrame.CraftingPage.SchematicForm.transaction.reagentTbls[1].reagentSlotSchematic.reagents);
            --DebugPrintTable(ProfessionsFrame.CraftingPage.SchematicForm.transaction.reagentTbls[1].reagentSlotSchematic.reagents[1]);
            
            -- 아래 2개 같은 결과(itemID 152512 모네라이트광석)
            --DebugPrintTable(ProfessionsFrame.CraftingPage.SchematicForm.transaction.reagentTbls[1].allocations.allocs[1].reagent);
            --DebugPrintTable(ProfessionsFrame.CraftingPage.SchematicForm.transaction.reagentTbls[1].reagentSlotSchematic.reagents[1]);
            --DebugPrintTable({GetItemInfo(152512)});

            --[[
                        local list = ProfessionsFrame.CraftingPage.SchematicForm.transaction.reagentTbls;
                        --DebugPrintTable(list);
                        --DebugPrintTable(#list);
                        for jdx = 1, #list do
                            --현재 선택된 레시피의 재료만 출력됨..실패
                            --DebugPrint( GetItemInfo(list[jdx].reagentSlotSchematic.reagents[1].itemID) );
                            
                            local transaction = ProfessionsFrame.CraftingPage.SchematicForm.transaction;
                            local reagentSlotSchematic = list[jdx].reagentSlotSchematic;

                            local qualityIndex = Professions.FindFirstQualityAllocated(transaction, reagentSlotSchematic) or 1;
                            local handled, link = Professions.HandleQualityReagentItemLink(recipeInfo.recipeID, reagentSlotSchematic, qualityIndex);
                            
                            if( handled ) then
                                DebugPrint("qualityIndex:"..tostring(qualityIndex));
                                DebugPrint("handled:"..tostring(handled));
                                DebugPrint("link:"..tostring(link));
                            end
                            
                        end
            --]]




            --DebugPrintTable(ProfessionsFrame.CraftingPage.SchematicForm.transaction.reagentTbls[2]);
            --DebugPrintTable(ProfessionsFrame.CraftingPage.SchematicForm.transaction.reagentTbls[2].allocations);
            --DebugPrintTable(ProfessionsFrame.CraftingPage.SchematicForm.transaction.reagentTbls[2].allocations.allocs);
            --DebugPrintTable(ProfessionsFrame.CraftingPage.SchematicForm.transaction.reagentTbls[2].reagentSlotSchematic);
            --DebugPrintTable(ProfessionsFrame.CraftingPage.SchematicForm.transaction.reagentTbls[2].reagentSlotSchematic.reagents);
            --DebugPrintTable(ProfessionsFrame.CraftingPage.SchematicForm.transaction.reagentTbls[2].reagentSlotSchematic.reagents[1].itemID);
            --DebugPrintTable({GetItemInfo(180733)});
        --end


        --
        
        
        -- 20221029 레시피갱신실패로 인해 저장하지 않도록 함.
        -- 10.0.0 API 삭제로 인해 레시피갱신실패
        -- GetRecipeNumReagents
        -- GetRecipeReagentInfo
        --if( 1 == 1 ) then return;end;
        
        --local numReagents = C_TradeSkillUI.GetRecipeNumReagents(recipeIDs[idx], nil);
        local numReagents = #recipeSchematic.reagentSlotSchematics;
        
        
        recipeInfo.reagentItemsCount = numReagents;  -- 재료아이템 갯수
        recipeInfo.reagentItems      = {};           -- 재료아이템 리스트
        for reagentIndex = 1, numReagents do
            --local reagentName, reagentTexture, reagentCount, playerReagentCount = C_TradeSkillUI.GetRecipeReagentInfo(recipeIDs[idx], reagentIndex);
            --print(reagentName);
            --recipeInfo.reagentItems[reagentIndex] = {
            --    ["reagentName"]        = reagentName or "",
            --    ["reagentTexture"]     = reagentTexture or "",
            --    ["reagentCount"]       = reagentCount or 0,
            --    ["playerReagentCount"] = playerReagentCount or 0,
            --    ["reagentLink"]        = C_TradeSkillUI.GetRecipeReagentItemLink(recipeIDs[idx], reagentIndex), -- 재료아이템링크
            --};
            local reagentSlotSchematic = recipeSchematic.reagentSlotSchematics[reagentIndex];
            local reagent = reagentSlotSchematic.reagents[1];
            
            --if( not reagent.itemID ) then
            --    DebugPrint(reagents);
            --end
            
            local item = Item:CreateFromItemID(reagent.itemID);
			local itemName = item:GetItemName(); --GetItemInfo(itemID);
            local itemLink = item:GetItemLink();
            local quantity = Professions.AccumulateReagentsInPossession(reagentSlotSchematic.reagents); -- 보유한 갯수
            local quantityRequired = reagentSlotSchematic.quantityRequired; -- 필요한 갯수

            recipeInfo.reagentItems[reagentIndex] = {
                ["reagentName"]        = itemName or "", --  or GetItemInfo(reagent.itemID)
                ["reagentTexture"]     = reagent.itemID or "",
                ["reagentCount"]       = quantityRequired or 0,
                ["playerReagentCount"] = quantity or 0,
                ["reagentLink"]        = itemLink or "", -- 재료아이템링크
            };
        end

        -- 전문기술에 별이 있는 경우 같은 기술이 3개가 된다.
        -- 별의 갯수와 재료의 종류와는 상관없으므로 다음단계가 없는 스킬만 추가한다.
        -- 즉, 별이 있는건 1개짜리만 저장 된다.
        if( recipeInfo["previousRecipeID"] ) then
            bSave = false;
        end;
        
        -- 재료템이 없는 전문기술 저장안함.
        if( recipeInfo.reagentItemsCount == 0 ) then
            bSave = false;
        end;
        
        -- 20221105
        -- 모든 재료템의 정보가 다 채워져 있지 않으면 레시피 저장안함
        if( recipeInfo.reagentItemsCount > 0 ) then
            for reagentIndex = 1, recipeInfo.reagentItemsCount do
                if( recipeInfo.reagentItems[reagentIndex].reagentName == "" ) then 
                    bSave = false;
                    break;
                end;
                if( recipeInfo.reagentItems[reagentIndex].reagentTexture == "" ) then 
                    bSave = false;
                    break;
                end;
            end
        end

        if( bSave ) then
            savedIndex = savedIndex + 1;
            --LJDProfessionDB[parentSkillLineID][savedIndex] = {};
            --LJDProfessionDB[parentSkillLineID][savedIndex] = recipeInfo;
            
            -- 20221103 savedIndex 대신 레시피ID 이용
            recipeInfo.IsRecipeIDSeparated = true;
            deleteList[recipeInfo.recipeID] = 1;    -- 값은 의미없음
            LJDProfessionDB[parentSkillLineID][recipeInfo.recipeID] = recipeInfo;
        end
        
        
        --]]
        
    end
    --DebugPrint("["..parentSkillLineID.."] 레시피 " .. tostring(#recipeIDs).."개중 "..tostring(savedIndex).. "개 저장완료!");
    
    -----------------------------------------------------------------------------------------------------------
    -- 10.0.0 부터 확장팩의 전문기술을 배우지 않은 경우 해당 확장팩의 레시피를 없애버리는 현상 발생
    -- 이를 피하기 위해 
    --  1. 삭제는 하지 않고, 갱신만하도록 함.
    --  2. 레시피ID별로 저장함. 기존 방식대로 인덱스로 저장하면 비교없이 덮어쓰므로..
    --
    -- 하지만, 동일 레시피가 여러개 표현될 수 있는 문제가 추가 발생함으로 해결하기 위한 중복제거 기능 필요.
    -----------------------------------------------------------------------------------------------------------
    
    -----------------------------------------------------------
    -- 중복 레시피 제거 기능
    -- ; Index별, 레시피ID별 중복시 레시피ID별 데이터만 유지
    -- ; 레시피ID별 데이터가 있는 경우 동일레시피 제거.
    -----------------------------------------------------------
    for recipeIdx, recipeList in pairs(LJDProfessionDB[parentSkillLineID]) do
        if( deleteList[recipeList.recipeID] ) then
            if( recipeList.IsRecipeIDSeparated ~= true ) then
                LJDProfessionDB[parentSkillLineID][recipeIdx] = nil;
            end
        end
    end
    
    --LJDProfessionDBChr = {}; -- 디버깅용
    --LJDProfessionDBChr = LJDProfessionDB[parentSkillLineID]; -- 디버깅용
    
end

-- 이름으로 재료템 찾기
function LJDFindTradeSkillReagentItems(itemName)
    
    local rtnString = {};
    local matchCount = 0;
    for skillLineID, skillLineList in LJDProfessionSortPairsByKeyTable(LJDProfessionDB) do  -- 20200102 정렬순서 지정
        --print(skillLineID);
        for recipeIdx, recipeList in pairs(skillLineList) do
            --print(recipeIdx);
            for reagentIdx, reagentList in pairs(recipeList["reagentItems"]) do
                --print(reagentIdx);
                -- 임시확인용 변수
                --LJDProfessionConfDB["TEMP"] = nil;
                --LJDProfessionConfDB["TEMP"] = LJDProfessionConfDB["TEMP"] or {};
                --LJDProfessionConfDB["TEMP"][recipeList.skillLineID] = LJDProfessionConfDB["TEMP"][recipeList.skillLineID] or {};
                --LJDProfessionConfDB["TEMP"][recipeList.skillLineID][recipeList.parentCategoryID or "nil"] = LJDProfessionConfDB["TEMP"][recipeList.skillLineID][recipeList.parentCategoryID or "nil"] or recipeList.parentCategoryName;
                
                -- 해당아이템을 사용하는 레시피 정보 출력
                if( itemName == reagentList["reagentName"] ) then
                    --print(reagentList["reagentLink"]);
                    --print(reagentList["reagentName"], reagentList["reagentLink"], string.match(reagentList["reagentLink"], "item[%-?%d:]+"));
                    matchCount = matchCount + 1;
                    rtnString[matchCount] = {
                        recipeList.skillLineID,          -- 1  
                        recipeList.skillLineName,        -- 2 
                        recipeList.name,                 -- 3 
                        recipeList.sourceType,           -- 4 
                        recipeList.disabled,             -- 5 
                        recipeList.reagentItemsCount,    -- 6 
                        recipeList.recipeID,             -- 7 
                        recipeList.numIndents,           -- 8 
                        recipeList.categoryID,           -- 9 
                        recipeList.parentCategoryID,     -- 10
                        recipeList.parentCategoryName,   -- 11
                        recipeList.learned,              -- 12
                        recipeList.icon,                 -- 13
                        recipeList.type,                 -- 14
                        recipeList.hiddenUnlessLearned,  -- 15
                        recipeList.craftable,            -- 16
                        recipeList.difficulty,           -- 17
                        recipeList.productQuality,       -- 18
                        recipeList.numAvailable,         -- 19
                        recipeList.favorite,             -- 20
                        recipeList.numSkillUps,          -- 21
                        reagentList.reagentName,         -- 22
                        reagentList.reagentLink,         -- 23
                        reagentList.playerReagentCount,  -- 24
                        reagentList.reagentTexture,      -- 25
                        reagentList.reagentCount,        -- 26
                    };
                    --DebugPrint(rtnString);
                    
                    -- 클래식에선 확장팩레시피 표현안함
                    if( isClassic and not LJDIsClassicCategory(recipeList["parentCategoryID"])) then
                        -- 이미 담은 레시피 제거
                        rtnString[matchCount] = nil;
                        matchCount = matchCount - 1;
                    end

                end
            end
        end
    end
    
    return unpack(rtnString);
end

-- 카테고리명이 전문기술명을 포함하고 있지 않으면 "카테고리-전문기술" 형태로 출력.
function LJDGetCategoryName( subCategoryName, skillLineName )
    local categoryName    = subCategoryName;
        
    -- 어떤 전문기술인지 category이름으로 알기 어렵다.
    -- 카테고리명이 전문기술명을 포함하고 있지 않으면
    -- "카테고리-전문기술" 형태로 출력.
    -- ex) 쿨 티란 디자인-보석세공, 잔달라 기계공학
    if not string.find(subCategoryName, skillLineName) then
        categoryName = subCategoryName.."-"..skillLineName.."";
    end
    return categoryName;
end

-- 아이템명으로 레시피정보를 찾아서 툴팁 마지막 라인에 출력.
function LJDAddRecipeinfoText( tooltip, itemName )

    local arrayTradeSkillReagentInfo = {LJDFindTradeSkillReagentItems(itemName)};
    --DebugPrint(arrayTradeSkillReagentInfo);

    if( #arrayTradeSkillReagentInfo > 0 ) then
      tooltip:AddLine(" ");
    end

    local summary = {};
    for i=1,#arrayTradeSkillReagentInfo do

        local tradeSkillReagentInfo = arrayTradeSkillReagentInfo[i];
        local summaryIndex = tradeSkillReagentInfo[1];
        local categoryID   = tradeSkillReagentInfo[10] or tradeSkillReagentInfo[9]; -- 상위카테고리ID
        local categoryName = LJDGetCategoryName(tradeSkillReagentInfo[11], tradeSkillReagentInfo[2]);

        summary[summaryIndex]                   = summary[summaryIndex] or {};
        summary[summaryIndex][categoryID]       = summary[summaryIndex][categoryID] or {};
        summary[summaryIndex][categoryID].count = summary[summaryIndex][categoryID].count or 0;
        summary[summaryIndex][categoryID].count = summary[summaryIndex][categoryID].count + 1;
        summary[summaryIndex][categoryID].sort  = summary[summaryIndex][categoryID].sort or i;  -- 정렬순서 용도(최초 사용 레시피의 index)
        --summary[summaryIndex][categoryID].ID    = categoryID;   -- 확장팩별 정렬시도(실패, 카테고리 ID 중구난방, 마법부여는 내림, 주문각인은 지맘대로)
        summary[summaryIndex][categoryID].name  = categoryName;

        -- 이하 디버그용
        --summary[summaryIndex][categoryID].name  = categoryName .. ":" .. tostring(summaryIndex) .. ":" .. tostring(categoryID);
        --summary[summaryIndex][categoryID].name  = categoryName .. ":" .. tostring(tradeSkillReagentInfo[10]) .. ":" .. tostring(tradeSkillReagentInfo[9]);
        
        -- 최대표현 갯수까지만 표현
        if( i <= MAX_DISPLAY_RECIPE_NUM ) then
            local difficulty      = tradeSkillReagentInfo[17];
            local difficultyColor = LJDTradeSkillTypeColor[difficulty];

            --------------------------------------------------------------------------------
            -- 찾는 레시피면 눈에 띄게(보라색)
            --------------------------------------------------------------------------------
            for i=1, #AlertRecipeList do
                if tradeSkillReagentInfo[3] == AlertRecipeList[i] then
                    ------------------------------------------------------------------------
                    -- 찾는 아이템의 difficulty 색과 같은 difficulty이면 모두 
                    -- 보라색이 되는 현상
                    -- 원인.
                    -- 진득주). TABLE 은 byRef 됨을 확인
                    -- [TABLE]difficultyColor 값을 변경하면 TradeSkillTypeColor도 바뀐다.
                    -- difficultyColor.r = 1;
                    -- difficultyColor.g = 0;
                    -- difficultyColor.b = 1;
                    -- 위처럼 테이블내값을 변경하면 안되고, 테이블 자체를 덮어씀
                    -- 어디까지 byRef 되는가.... 테이블을 덮어쓰는건 영향없었음.
                    ------------------------------------------------------------------------
                    difficultyColor = LJDTradeSkillTypeColor.AlertRecipe;
                    break;
                end;
            end

            tooltip:AddDoubleLine(
                categoryName, 
                tradeSkillReagentInfo[3],
                categoryColor.r, categoryColor.g, categoryColor.b, 
                difficultyColor.r, difficultyColor.g, difficultyColor.b
            );
        end
    end

    -- 최대표현 갯수까지만 표현하므로 전체 갯수 표현.
    if( #arrayTradeSkillReagentInfo > MAX_DISPLAY_RECIPE_NUM ) then
        tooltip:AddLine("...", categoryColor.r, categoryColor.g, categoryColor.b);
        tooltip:AddLine("--------------------------", categoryColor.r, categoryColor.g, categoryColor.b);
     
        -- summary 레시피 출력순서와 동일하게 하기
        --for skillLineID, skillLineList in pairs(summary) do
        for skillLineID, skillLineList in LJDProfessionSortPairsByKeyTable(summary) do
            --for k, v in pairs(skillLineList) do
            for k, v in LJDProfessionSortPairs(skillLineList, function(t,a,b) return t[b].sort > t[a].sort end) do    -- 레시피Index의 오름차순(최근 확장팩이 젤위)
                tooltip:AddDoubleLine(v.name, tostring(v.count).." "..unitsOfNumber, categoryColor.r, categoryColor.g, categoryColor.b, categoryColor.r, categoryColor.g, categoryColor.b);
            end
        end
        
        --tooltip:AddLine("--------------------------");
        --tooltip:AddDoubleLine("전체", tostring(#arrayTradeSkillReagentInfo).." 개", _, _, _, _, _, _);
    end
    
    tooltip:Show();

end

--------------------------------------------------------------------------
-- 아이템 툴팁(가방, 은행등)
--------------------------------------------------------------------------
function on_GameTooltip_SetItem(self, ...)
    local ll = {...};
    --DebugPrint(self:GetName());
    --DebugPrint(ll);
    --DebugPrint(ll);
    --DebugPrintTable(self);

    local tooltipData = self:GetTooltipData();
    if not tooltipData then
        return;
    end

    if ( not tooltipData.hyperlink ) and ( not tooltipData.guid ) then
        return;
    end

    local hyperlink = tooltipData.hyperlink or C_Item.GetItemLinkByGUID(tooltipData.guid);
    if not hyperlink then
        return;
    end

    --local itemName, itemLink = self:GetItem();
    --20221119 10.0.2 대응
    --local itemName, itemLink = self.GetItem and self:GetItem();
    --20221120 10.0.2 대응 2
    local itemLink = hyperlink;
    local itemName = GetItemInfo(hyperlink);


--[[
--TooltipUtil.lua
function TooltipUtil.GetDisplayedItem(tooltip)
	if tooltip:IsTooltipType(Enum.TooltipDataType.Item) then
		local tooltipData = tooltip:GetTooltipData();
		local hyperlink;
		if tooltipData.guid then
			hyperlink = C_Item.GetItemLinkByGUID(tooltipData.guid);
		elseif tooltipData.hyperlink then
			hyperlink = tooltipData.hyperlink;
		end
		if hyperlink then
			local name = GetItemInfo(hyperlink);
			return name, hyperlink, tooltipData.id;
		end
	end
end
--]]



    -- 재료템 사용 레시피
    LJDAddRecipeinfoText( self, itemName );
    
    -- 설정값에 의한 보유갯수 표현여부
    if LJDProfessionConfDB["IsShowNumberOfItems"] then
        -----------------------------------------
        -- 20191205
        -- 현 케릭의 은행포함 보유갯수 출력 추가
        -- 형식) [케릭명] : [보유갯수] 개
        -----------------------------------------
        local itemCount = GetItemCount(itemLink, true);   -- 은행포함.(2번째 인자)

        -- "내 케릭명" 글자색을 케릭클래스색으로 표현
        local unitId = "player";
        local className, classFilename, classID = UnitClass(unitId);
        local rPerc, gPerc, bPerc, classColorArgbHex = GetClassColor(classFilename);
        local unitName = UnitName(unitId);

        -- 툴팁의 라인수 추가가 부담되어 오른쪽 상단에 표현함.
        -- GameTooltipTemplate.xml 파일에서 오른쪽 상단 FontString 있음을 확인함.
        local objItemCount = _G[self:GetName().."TextRight1"];
        if( objItemCount ) then
            objItemCount:SetText("|c"..classColorArgbHex..""..unitName.."".."|r|c"..countColorArgbHex.." : "..itemCount.." "..unitsOfNumber.."|r");
            objItemCount:Show(); -- 기본숨김처리 되어있음.
        end

        self:Show();
    end
end
--GameTooltip:HookScript("OnTooltipSetItem", on_GameTooltip_SetItem);
--ItemRefTooltip:HookScript("OnTooltipSetItem", on_GameTooltip_SetItem);  -- 화면 비좁음으로 주석처리함.
-- 20221119 10.0.2 대응
TooltipDataProcessor.AddTooltipPostCall(Enum.TooltipDataType.Item, on_GameTooltip_SetItem);

--------------------------------------------------------------------------
-- 전문기술창의 재료아이템 툴팁
--------------------------------------------------------------------------
function on_GameTooltip_SetRecipeReagentItem(self, ...)
    local ll = {...};
    --DebugPrint(ll);
    --DebugPrintTable(self);

    local recipeID, reagentIndex = ...;

    local recipeInfo = C_TradeSkillUI.GetRecipeInfo(recipeID);
    
    --local reagentName, reagentTexture, reagentCount, playerReagentCount = C_TradeSkillUI.GetRecipeReagentInfo(recipeID, reagentIndex);
    --local itemID = ProfessionsFrame.CraftingPage.SchematicForm.transaction.reagentTbls[reagentIndex].reagentSlotSchematic.reagents[1].itemID;
    local recipeSchematic = C_TradeSkillUI.GetRecipeSchematic(recipeID, false);
    local reagentSlotSchematic = recipeSchematic.reagentSlotSchematics[reagentIndex];
    if( not reagentSlotSchematic ) then
        --DebugPrint(recipeID);
        --DebugPrint(reagentIndex);
        --DebugPrint(reagentSlotSchematic);
        return;
    end
    
    local reagent = reagentSlotSchematic.reagents[1];
    local itemID  = reagent.itemID;
    local reagentName = GetItemInfo(itemID);
    
--DebugPrint(recipeInfo.relativeDifficulty);

    --[[
    DebugPrintTable(recipeSchematic);
    DebugPrintTable(recipeSchematic.reagentSlotSchematics);
    DebugPrintTable(reagentSlotSchematic);
    DebugPrintTable(reagent);
    DebugPrintTable(itemID);
    DebugPrintTable(reagentName);
    
    if reagent.itemID then
        local item = Item:CreateFromItemID(reagent.itemID);
        local itemName = item:GetItemName(); -- 아이템명(재료)
        local quantity = Professions.AccumulateReagentsInPossession(reagentSlotSchematic.reagents); -- 보유한 갯수
        local quantityRequired = reagentSlotSchematic.quantityRequired; -- 필요한 갯수
        local text = PROFESSIONS_TRACKER_REAGENT_FORMAT:format(PROFESSIONS_TRACKER_REAGENT_COUNT_FORMAT:format(quantity, quantityRequired), itemName)

        -- Ex) 보유갯수/필요갯수 
        -- 아이템명 2/5 대해 메기
        DebugPrint(text);

    end
    --]]
    
    --if( 1 == 1 ) then return; end;
    
    LJDAddRecipeinfoText( self, reagentName );

    ---------------------------------------------------------------------------
    -- 전문기술창의 툴팁이 내용이 길어지면서 아이콘을 가리게 되는 불편함 발견.
    -- 툴팁의 위치를 아이콘의 오른쪽 아래로 옮긴다.
    ---------------------------------------------------------------------------
    local point, relativeTo, relativePoint, xOfs, yOfs = self:GetPoint();
    self:ClearAllPoints();
    --self:SetPoint("TOPLEFT", TradeSkillFrame, "TOPRIGHT", xOfs, yOfs);
    self:SetPoint("TOPLEFT", relativeTo, "BOTTOMRIGHT", xOfs, yOfs);
    
    --[[
    -- 이하 같은 기능 어려운 방법
    -- 전문기술창에서 재료아이템 버튼을 찾은후 anchor .. 
    local reagentButton = TradeSkillFrame.DetailsFrame.Contents.Reagents[reagentIndex];
    DebugPrintTable(reagentButton);
    self:SetPoint("BOTTOMLEFT", reagentButton, "TOPRIGHT");
    --]]
    ---------------------------------------------------------------------------
    
end

--[[
-- 20221119 10.0.2 대응
if( GameTooltip["SetRecipeReagentItem"] ) then
    hooksecurefunc(GameTooltip, 'SetRecipeReagentItem', on_GameTooltip_SetRecipeReagentItem);
end
--]]

--시도1
function LJDHookfnc()
    -- 전문 기술창
    -- 재료템 아이콘위에 마우스를 가져갔을때 처리 함수를 후킹한다.
    
    -- 후킹전 함수ID
    -- ( Frame과 Mixin의 함수ID가 같다.)
    DebugPrint(TradeSkillFrame.DetailsFrame.OnReagentMouseEnter);
    DebugPrint(TradeSkillDetailsMixin.OnReagentMouseEnter);

    -- override
    --TradeSkillDetailsMixin.OnReagentMouseEnter = LJDTradeSkillDetails_OnReagentMouseEnter;
    --TradeSkillFrame.DetailsFrame.OnReagentMouseEnter = LJDTradeSkillDetails_OnReagentMouseEnter;
    
    -- Hook
    hooksecurefunc(TradeSkillFrame.DetailsFrame, 'OnReagentMouseEnter', LJDTradeSkillDetails_OnReagentMouseEnter);

    -- 후킹후 함수ID
    -- ( Mixin의 함수ID는 변함없지만, Frame의 함수ID는 변경되었다.)
    -- ; 기존함수(OnReagentMouseEnter) + 
    --   후킹함수(LJDTradeSkillDetails_OnReagentMouseEnter)를 합쳐서
    --   새로운 함수로 저장하는듯
    DebugPrint(TradeSkillFrame.DetailsFrame.OnReagentMouseEnter);
    DebugPrint(TradeSkillDetailsMixin.OnReagentMouseEnter);
end

--시도2
function LJDTradeSkillDetails_OnReagentMouseEnter(self, reagentButton)
    -- [Blizzard_TradeSkillDetails.xml] 에서 참조함.
    -- TradeSkillFrame.DetailsFrame.Contents.Reagent1 ~ 8
    -- 둘다 TradeSkillFrame 출력함.
    DebugPrint(self:GetParent():GetName());
    DebugPrint(reagentButton:GetParent():GetParent():GetParent():GetName());
    
    GameTooltip:SetOwner(reagentButton, "ANCHOR_TOPRIGHT");
    GameTooltip:SetRecipeReagentItem(self.selectedRecipeID, reagentButton.reagentIndex);
    CursorUpdate(reagentButton);
end
--[[
function TradeSkillDetailsMixin:OnReagentMouseEnter(reagentButton)
    GameTooltip:SetOwner(reagentButton, "ANCHOR_TOPLEFT");
    GameTooltip:SetRecipeReagentItem(self.selectedRecipeID, reagentButton.reagentIndex);
    CursorUpdate(reagentButton);
end
--]]

-- 클래식 레시피인지 반환(상위카테고리ID로 판단)
function LJDIsClassicCategory(parentCategoryID)
    if( not parentCategoryID ) then
        return false;
    end

    local classicCategoryIdList = {
--        [415] = "주문각인",
        [72] = "구세계 요리법",
        [604] = "연금술",
        [590] = "대장기술 도면",
        [419] = "기계공학",
        [379] = "가죽세공 도안",
        --[372] = "보석세공 디자인",
        [362] = "재봉술 도안",
        [667] = "마법부여",
        [1078] = "채광",
    };

    return (classicCategoryIdList[parentCategoryID] ~= nil );
end

